# -*- coding: utf-8 -*-
import os
from shutil import rmtree
from tempfile import mkdtemp

import sh


class Increment(object):
    def __init__(self, timestamp):
        self.timestamp = timestamp

    @staticmethod
    def from_string(line):
        timestamp = int(line.strip().split()[0])
        return Increment(timestamp)


class RdiffAPI(object):
    def __init__(self, rsync_dir, tmp_dir=None, disable_compression=False):
        self.rsync_dir = rsync_dir

        self._tmp_dir = tmp_dir
        if self._tmp_dir is None:
            self._tmp_dir = mkdtemp()

        self._disable_compression = disable_compression

        self._options = []

    def yield_increments(self):
        increments = sh.rdiff_backup("--parsable-output", "-l", self.rsync_dir)

        for increment_line in increments:
            yield Increment.from_string(increment_line)

    def restore(self, out_dir, time):
        sh.rdiff_backup("-r", time, self.rsync_dir, out_dir)

    def add_increment(self, from_dir, timestamp=0):
        options = self._options[:]
        if timestamp != 0:
            options.extend(("--current-time", timestamp))

        if self._disable_compression:
            options.append("--no-compression")

        options.append(from_dir)
        options.append(self.rsync_dir)

        sh.rdiff_backup(*options)

    def restore_into(self, out_dir, increments_to_keep,
                     disable_compression=False):
        out_rsync = RdiffAPI(out_dir, disable_compression=disable_compression)
        for increment in sorted(increments_to_keep, key=lambda x: x.timestamp):
            rmtree(self._tmp_dir)
            os.mkdir(self._tmp_dir)

            print "Restoring", increment.timestamp

            self.restore(self._tmp_dir, increment.timestamp)
            out_rsync.add_increment(self._tmp_dir, increment.timestamp)

        rmtree(self._tmp_dir)
        return out_rsync



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
from setuptools import setup
from setuptools import find_packages

from docs import get_version


CHANGELOG = open('CHANGES.rst').read()
LONG_DESCRIPTION = "\n\n".join([
    open('README.rst').read(),
    CHANGELOG
])


setup(
    name='rdiff_trimmer',
    version=get_version(CHANGELOG),
    py_modules=['rsync_trimmer'],

    author='Bystroushaak',
    author_email='bystrousak@kitakitsune.org',

    url='https://github.com/Bystroushaak/rdiff_trimmer',
    license='MIT',
    description='Get rid of the old rdiff-backup increments from your backup.',

    long_description=LONG_DESCRIPTION,

    packages=find_packages('src'),
    package_dir={'': 'src'},

    install_requires=open("requirements.txt").read().splitlines(),
    include_package_data=True,

    classifiers=[
        "License :: OSI Approved :: MIT License",

        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],

    scripts=[
        'bin/rdiff_trimmer',
        'bin/unpack_rdiff_increments',
    ],

    extras_require={
        "test": [
            "pytest",
            "pytest-cov",
        ],
        "docs": [
            "sphinx",
            "sphinxcontrib-napoleon",
        ]
    }
)
